--[[
sample file and logger documentation.
the logger helps the coder to determine what's happening with your code, on the transmitter.
It works by providing the coder with two entry points: 
log() --> this is the main function to be used in your code, with a list of parameters that will drive the log actions
events --> this is the list of supported events; these are:
configure --> used to configure, should be used at the very first lines of the code. Please look at the event, at the end of this sample
configureTelemetry --> this is a special configuration for the variable watches. It can only be used within the "init" function or afterwards.
                       it's meant to be used after the log activation.
activate --> used to activate the logger. This is done in order to quickly activate or deactivate the logger,
             without the need to replace all the log lines.
key --> this is a special event, used to log a key event.
open and close --> these two events are to log when a function starts or stops, or when a specific event needs to be grouped.
any other events are logged but no special action is available.

the location of the log file is determined during configuration and is stored in a subfolder called Debug
the log file structure is XML-like, and if you process the log file with an XML beautifier, you should be able to see the nested calls.  

the logger and this sample are provided WITHOUT WARRANTY, and on a best effort base. No responsabilty is derived.

please contact me thru the jetiforum.de site: user ABIEF



to quickly comment the logger lines when deploying production code, replace
logger.log({
with
--logger.log({
reverse this action when wanting to reactivate your log.
 remember to comment or uncomment the line below:
 --]]
local logger = require("Logger") -- remove in production code

local appName = "Sample" 
local appVersion = "1.01a"
local supportedFirmwareVersion = "4.22"
local rootFolder = "Apps/Sample"

local function keyEvent(key)
  -- when logging a function, is good practice to log both its start and its end. for this you use the events open and close
  logger.log({functionName = "keyEvent", event = logger.events.open})
  -- this logs a special entry, detailing the key
  logger.log({key = key, event=logger.events.key})
  -- add your code here
  logger.log({functionName = "keyEvent", event = logger.events.close})
end

local function initForm(subformID)
  logger.log({functionName = "initForm", value = subformID, event = logger.events.open})
  -- add your code here
  logger.log({functionName = "initForm", value = subformID, event = logger.events.close})
end

local function loopForm()
  logger.log({functionName = "loopForm", event = logger.events.open})
  -- add your code here.
  -- for performance issues, it is recommended not to use the logger on fast loops Activate only when necessary
  logger.log({functionName = "loopForm", event = logger.events.close})
end

local function closeForm()
  logger.log({functionName = "closeForm", event = logger.events.open})
  -- add your code here
  logger.log({functionName = "closeForm", event = logger.events.close})
end

local function loop()
  logger.log({functionName = "loop", event = logger.events.open})
  -- add your code here.
  -- for performance issues, it is recommended not to use the logger on fast loops Activate only when necessary
  
  logger.log({windowNo = 0, telemetryPairs = {}})
  -- use this to record the variables to watch. If windowNo is 0, these will get assigned to the first telemetry window available
  -- in telemetryPairs, you declare a list of variables and values that you want to watch on the telemetry screen.

  collectgarbage()
  logger.log({functionName = "loop", event = logger.events.close})
end

local function init()
  logger.log({functionName = "init", event = logger.events.open})
  -- add your code here
  
  -- this will generate an entry in the log with the defined variable.
  -- supported variables are:
  -- any variable (boolean, int, string, ..)
  -- any table of variables: this get broken down into the single elements
  -- switch userdata: this gets broken down into all the elements of the switch
  -- if a table contains a switch, it also gets broken down in all the elements of the switch
  logger.log({rootFolder = rootFolder})
  
  -- you can only configure the watch telemetry in the init function
  -- use the flags telemetryScreen1 or telemetryScreen2 to define the telemetry screen to activate
  logger.log({event = logger.events.configureTelemetry, telemetryScreen1 = true, telemetryScreen2 = false})
  
  -- add your code here
  system.registerForm(1, MENU_MAIN, appName, initForm, keyEvent, loopForm, closeForm)
  
  collectgarbage()
  logger.log({functionName = "init", event = logger.events.close})
end

-- pre init
collectgarbage()
if (system.getVersion() < supportedFirmwareVersion) then
  system.messageBox("FATAL"..": "..string.format("this code is for %s firmware version or above", supportedFirmwareVersion))
  return {init = nil, loop = nil, author = nil, version = nil, name = nil}
else
  -- here is where you configure the logger. You need to first configure it and then activate it. 
  -- if not activated, the logger will not work.
  -- to configure, you need to send both the application name, the root foler where to store the log and 
  -- the filename that will be used to store the log. 
  -- the file will always have .log as extension.
  logger.log({event = logger.events.configure, application = appName, folder = rootFolder, filename = appName})
  logger.log({event = logger.events.activate})

  return {init = init, loop = loop, author = "rgfacchini", version = appVersion, name = appName}
end